@@ -24,6 +24,7 @@
 #include <signal.h>
 
 JSValueRef seed_print_ref;
+JSValueRef seed_printerr_ref;
 
 static JSValueRef
 seed_include (JSContextRef ctx,
@@ -230,6 +231,31 @@ seed_print (JSContextRef ctx,
   return JSValueMakeUndefined (ctx);
 }
 
+static JSValueRef
+seed_printerr (JSContextRef ctx,
+	       JSObjectRef function,
+	       JSObjectRef this_object,
+	       size_t argumentCount,
+	       const JSValueRef arguments[],
+	       JSValueRef *exception)
+{
+  gchar *buf;
+  if (argumentCount != 1)
+    {
+      seed_make_exception (ctx, exception, "ArgumentError",
+			   "printerr expected 1 argument, got %zd",
+			   argumentCount);
+      return JSValueMakeNull (ctx);
+    }
+
+  buf = seed_value_to_string (ctx, arguments[0], exception);
+
+  g_printerr ("%s\n", buf);
+  g_free (buf);
+
+  return JSValueMakeUndefined (ctx);
+}
+
 const gchar *
 seed_g_type_name_to_string (GITypeInfo * type)
 {
@@ -301,15 +327,31 @@ seed_introspect (JSContextRef ctx,
   for (i = 0; i < nargs; ++i)
     {
       argument = JSObjectMake (ctx, NULL, NULL);
+      GIArgInfo* arg_info = g_callable_info_get_arg (info, i);
+      const gchar *arg_type =	seed_g_type_name_to_string (
+			g_arg_info_get_type(arg_info));
+      const gchar *arg_name =	g_base_info_get_name((GIBaseInfo*) arg_info);
+      GIDirection dir = g_arg_info_get_direction (arg_info);
 
-      const gchar *arg_name =
-	seed_g_type_name_to_string (g_arg_info_get_type
-				    (g_callable_info_get_arg (info, i)));
-
+     
       seed_object_set_property (ctx, argument, "type",
+				seed_value_from_string (ctx,
+							arg_type, exception));
+      seed_object_set_property (ctx, argument, "name",
 				seed_value_from_string (ctx,
 							arg_name, exception));
 
+      seed_object_set_property (ctx, argument, "allow_none",
+			seed_value_from_boolean (ctx,
+				g_arg_info_may_be_null (arg_info) ? 1 : 0, exception));
+
+
+      seed_object_set_property (ctx, argument, "direction",
+				seed_value_from_string (ctx,
+					dir == GI_DIRECTION_OUT ? "out" :
+						(dir == GI_DIRECTION_IN ? "in" : "inout")
+					, exception));
+
       JSObjectSetPropertyAtIndex (ctx, args_obj, i, argument, NULL);
     }
 
@@ -487,6 +529,13 @@ seed_init_builtins (SeedEngine * local_eng, gint * argc, gchar *** argv)
 			    seed_print_ref);
   JSValueProtect (local_eng->context, seed_print_ref);
 
+  seed_printerr_ref =
+    JSObjectMakeFunctionWithCallback (local_eng->context, NULL, &seed_printerr);
+  seed_object_set_property (local_eng->context, obj, "printerr", seed_printerr_ref);
+  seed_object_set_property (local_eng->context, local_eng->global, "printerr",
+			    seed_printerr_ref);
+  JSValueProtect (local_eng->context, seed_printerr_ref);
+
   seed_create_function (local_eng->context,
 			"check_syntax", &seed_check_syntax, obj);
   seed_create_function (local_eng->context,
